/*******************************************************************
 *
 *	File:		Dialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Dialogs.h"
#include "controls.h"
#include "CSimpleTextParser.h"

#include <emame.mbg>

#include "emameapp.h"
#include "version.h"

extern "C"
{
#include "audit.h"
}


#include <txtrich.h>				// CRichText
#include <eikrted.h>				// CEikRichTextEditor


/************************************************
 *
 * GameRunning Dialog
 *
 ************************************************/

CGameRunningDialog::CGameRunningDialog(const TDesC8& aGameName, CMameEngine& aMameEngine) : iMameEngine(aMameEngine)
{
	iGameName.Copy(aGameName);
}

void CGameRunningDialog::PreLayoutDynInitL()
{
	CEikLabel*	label =(CEikLabel*)Control(ECtlRunningGameName);
	label->SetTextL(iGameName);
}

void CGameRunningDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case ECtlCmdButPanelRunningGame:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlCmdButStopGame:
					iMameEngine.ForceGameExit();
					iMameEngine.ResumeGame();
					break;

				case ECtlCmdButResumeGame:
					iMameEngine.ResumeGame();
					break;
				default:
					break;
			}
			break;
		}
		default:
			break;
	}
}

void CGameRunningDialog::SetSizeAndPositionL(const TSize& aSize)
{
	SetCornerAndSize(EHCenterVCenter, aSize);
}



/************************************************
 *
 * About Dialog
 *
 ************************************************/

CAboutDialog::CAboutDialog() : CSimpleTextInfoDialog(-1, ECtlAboutVersion, R_SIMPLE_TEXT_ABOUT_LINKS, ECtlAboutLinks)
{
}

void CAboutDialog::ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId)
{
	if (aRichTextCtlId == ECtlAboutLinks)
		CSimpleTextInfoDialog::ShowTextL(aRichText, aRichTextCtlId, aResId);
	else
	{
		TBuf<16>	versionText;
		TBuf<512>	text;

#if (EMAME_VERSION_PATCH > 0)
		versionText.Format(_L("%d.%d.%d"), EMAME_VERSION_MAJOR, EMAME_VERSION_MINOR, EMAME_VERSION_PATCH);
#else
		versionText.Format(_L("%d.%d"), EMAME_VERSION_MAJOR, EMAME_VERSION_MINOR);
#endif

		HBufC*		aboutFormat =	CEikonEnv::Static()->AllocReadResourceLC(R_SIMPLE_TEXT_ABOUT);
		text.Format(*aboutFormat, &versionText);

		ShowSimpleTextL(text, aRichText);

		CleanupStack::PopAndDestroy(aboutFormat);
	}
}

/************************************************
 *
 * Game has no rom Dialog
 *
 ************************************************/

CGameHasNoRomDialog::CGameHasNoRomDialog(const TDesC& aName, const TDesC& aGameShortName) : CSimpleTextInfoDialog(-1, ECtlAboutVersion),
	iGameName(aName), iGameShortName(aGameShortName)
{
}

void CGameHasNoRomDialog::ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId)
{
	if (aRichTextCtlId == 1)
	{
		TBuf<1024>	text;

		HBufC*		format =	CEikonEnv::Static()->AllocReadResourceLC(R_SIMPLE_TEXT_GAME_HAS_NO_ROM);
		text.Format(*format, &iGameName, &iGameShortName);

		ShowSimpleTextL(text, aRichText);

		CleanupStack::PopAndDestroy(format);
	}
}



/************************************************
 *
 * Scanning ROMs Dialog
 *
 ************************************************/


CScanningRomDialog::CScanningRomDialog(CMameEngine& aMameEngine, CInstalledGameList& aGameList)
	: iMameEngine(aMameEngine), iInstalledGameList(aGameList)
{
}

CScanningRomDialog::~CScanningRomDialog()
{
	delete iIdle;
}

void CScanningRomDialog::PreLayoutDynInitL()
{
	iProgress = (CProgressControl*)Control(ECtlScanningRomProgress);

	iIdle = CIdle::NewL(CActive::EPriorityIdle);
	iIdle->Start(TCallBack(&CScanningRomDialog::IdleCallBack, this));

	iNoOfGames = 0;
	const struct GameDriver**	game = iMameEngine.GameDrivers();
	while(*game++)
		iNoOfGames++;
}

void CScanningRomDialog::PostLayoutDynInitL()
{
	TInt height = ControlCaption(ECtlScanningRomProgress)->Size().iHeight + 4;
	TSize	size = iProgress->Size();
	iProgress->SetSize(TSize(size.iWidth, height));

	TInt y = ControlCaption(ECtlScanningRomProgress)->Position().iY;
	TPoint	pos = iProgress->Position();
	iProgress->SetPosition(TPoint(pos.iX, y - 2));
}


TInt CScanningRomDialog::IdleCallBack(TAny* aParam)
{
	return ((CScanningRomDialog*) aParam)->DoUpdate();
}

TInt CScanningRomDialog::DoUpdate()
{
	TInt missing = RomsetMissing(iGameNo);
	if (!missing)
		iInstalledGameList.AddGame(iGameNo);

	iGameNo++;
	UpdateProgress();

	TInt more = (iGameNo < iNoOfGames) ? 1 : 0;
	if (!more)
	{
		TRAPD(err, TryExitL(EEikBidOk));
	}
	return more;
}

void CScanningRomDialog::UpdateProgress()
{
	iProgress->SetProgress((iGameNo * 100) / iNoOfGames);
}


SEikControlInfo CScanningRomDialog::CreateCustomControlL(TInt aControlType)
{
	SEikControlInfo controlInfo;

	controlInfo.iControl = NULL;
	controlInfo.iTrailerTextId = 0;
	controlInfo.iFlags = 0;

	if(aControlType == EProgressCtlType)
	{
		CCoeControl* control = new(ELeave)CProgressControl;
		controlInfo.iControl = control;
	}

	return(controlInfo);
}


/************************************************
 *
 * Game Info Dialog
 *
 ************************************************/

CGameInfoDialog::CGameInfoDialog(CMameEngine& aMameEngine, TInt aGame) 
	: CSimpleTextInfoDialog(-1, ECtlGameInfoEdwin) , iMameEngine(aMameEngine), iGame(aGame)
{
}

void CGameInfoDialog::AppendTextL(TDes& aOutput, TInt aResId, const TDesC& aGameText)
{
	TBuf<256>	text;
	HBufC*		format =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
	text.Format(*format, &aGameText);
	aOutput.Append(text);
	CleanupStack::PopAndDestroy(format);
}

void CGameInfoDialog::ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt /*aResId*/)
{
	if (aRichTextCtlId == ECtlGameInfoEdwin)
	{
		TBuf8<64>	text8;
		TBuf<64>	text;
		TBuf<512>	fullText;
		
		const struct GameDriver*	game = *(iMameEngine.GameDrivers() + iGame);

		text8.Format(_L8("%s (%s)"), game->description, (const TUint8*)game->name);
		text.Copy(text8);
		AppendTextL(fullText, R_GAME_INFO_TITLE, text);

		if (iMameEngine.IsGameCloneOf(game))
		{
			text8.Format(_L8("%s"), game->clone_of->description);
			text.Copy(text8);
			AppendTextL(fullText, R_GAME_INFO_CLONE, text);
		}

		text8.Format(_L8("%s, %s"), game->year, game->manufacturer);
		text.Copy(text8);
		AppendTextL(fullText, R_GAME_INFO_MANUFACTURED, text);

		text8.Format(_L8("%d x %d, using %d colors"), game->drv->screen_width, game->drv->screen_height, game->drv->total_colors);
		text.Copy(text8);
		AppendTextL(fullText, R_GAME_INFO_RESOLUTION, text);

		float	freq = game->drv->cpu[0].cpu_clock;
		freq /= (1000 * 1000);
		text8.Format(_L8("%s @ %.2f MHz"), iMameEngine.CpuName(game->drv->cpu[0].cpu_type), freq);
		text.Copy(text8);
		AppendTextL(fullText, R_GAME_INFO_CPU, text);

		TInt missing = RomsetMissing(iGame);
		HBufC*	romText = CEikonEnv::Static()->AllocReadResourceLC(missing ? R_GAME_INFO_ROMS_MISSING : R_GAME_INFO_ROMS_PRESENT);
		fullText.Append(*romText);
		CleanupStack::PopAndDestroy(romText);

		ShowSimpleTextL(fullText, aRichText);
	}
}




/************************************************
 *
 * Game Options Dialog
 *
 ************************************************/


void CGameOptionsDialog::PreLayoutDynInitL()
{
	CEikChoiceList* choiceList;

	CEikCheckBox* checkBox=(CEikCheckBox*)Control(ECtlGameOptionsSound);
	checkBox->SetState(iGameOptions.iSoundEnabled ? CEikButtonBase::ESet : CEikButtonBase::EClear);

	choiceList=(CEikChoiceList*)Control(ECtlGameOptionsGameSize);
	choiceList->SetCurrentItem((TInt) iGameOptions.iGameSize);
}



void CGameOptionsDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	if (size.iWidth < 400)
		size.iWidth = 400;
	SetCornerAndSize(EHCenterVCenter, size);
}

    
TBool CGameOptionsDialog::OkToExitL(TInt aKeycode)
{
	CEikChoiceList* choiceList;

	CEikCheckBox* checkBox=(CEikCheckBox*)Control(ECtlGameOptionsSound);
	iGameOptions.iSoundEnabled = (checkBox->State() == CEikButtonBase::ESet) ? (TBool) ETrue : (TBool) EFalse;

	choiceList=(CEikChoiceList*)Control(ECtlGameOptionsGameSize);
	iGameOptions.iGameSize = (TGameOptions::TGameSize) choiceList->CurrentItem();

	return ETrue;
}


/************************************************
 *
 * Play List Dialog
 *
 ************************************************/


void CPlayListDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize dialogSize(400, aSize.iHeight);
	SetCornerAndSize(EHCenterVCenter,dialogSize);
}


void CPlayListDialog::ShowGameDetailsL(TInt aGame)
{
	CGameInfoDialog*	dialog = new CGameInfoDialog(iMameEngine, aGame);
	dialog->ExecuteLD(R_DIALOG_GAME_INFO);
}

void CPlayListDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId)
	{
		case ECtlCmdButPanel:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlCmdButRun:
					TryExitL(EEikBidOk);
					break;
				case ECtlCmdButCancel:
					TryExitL(EEikBidCancel);
					break;
				case ECtlCmdButInfo:
					ShowGameDetailsL(GameIndex());
					break;
			}
			break;
		}
	}
}

void CPlayListDialog::PreLayoutDynInitL()
{
	TBuf8<128>	gameName8;
	TBuf<128>	gameName;

	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlListboxPlayList);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());

	// append contents of the model's checksum array to listbox for display
	TInt count = NoOfGames();
	for (TInt i=0 ; i<count ; i++)
	{
		TInt gameNo = GameNo(i);

		const struct GameDriver*	game = iMameEngine.GameDrivers()[gameNo];
		gameName8.Format(_L8("%s"), game->description);
		gameName.Copy(gameName8);
		listboxArray->AppendL(gameName);
		game++;
	}
	listboxArray->Sort();


	CEikScrollBarFrame* scrollbarFrame = listbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);

	SetTitleL(iTitleResId);
}

void CPlayListDialog::PostLayoutDynInitL()
{
	TBuf8<128>	gameName8;
	TBuf<128>	gameName;

	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlListboxPlayList);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());

	if (iGameToRun != -1)
	{
		TInt	index = 0;
		gameName8.Format(_L8("%s"), (*(iMameEngine.GameDrivers() + iGameToRun))->description);
		gameName.Copy(gameName8);
		if (listboxArray->FindIsq(gameName, index) == KErrNone)
		{
			listbox->HandleItemAdditionL();
			listbox->SetCurrentItemIndex(index);
			listbox->View()->ScrollToMakeItemVisible(index);
			listbox->UpdateScrollBarsL();
		}
	}
}

TInt CPlayListDialog::GameIndex()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlListboxPlayList);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());
	return iMameEngine.GameNo(listboxArray->operator[](listbox->CurrentItemIndex()));
}

TBool CPlayListDialog::OkToExitL(TInt /*aKeycode*/)
{
	TBuf<128>	gameName;
	iGameToRun = GameIndex();
    return(ETrue);
}


/************************************************
 *
 * All games play List Dialog
 *
 ************************************************/

TInt CAllGamesPlayListDialog::NoOfGames()
{
	return iMameEngine.NoOfGames();
}

TInt CAllGamesPlayListDialog::GameNo(TInt aIndex)
{
	return aIndex;
};

/************************************************
 *
 * Installed games play List Dialog
 *
 ************************************************/

TInt CInstalledGamesPlayListDialog::NoOfGames()
{
	return iMameEngine.InstalledGameList().NoOfGames();
}

TInt CInstalledGamesPlayListDialog::GameNo(TInt aIndex)
{
	return iMameEngine.InstalledGameList().GameNo(aIndex);
};



/************************************************
 *
 * SimpleTextInfoDialog
 *
 ************************************************/


CSimpleTextInfoDialog::CSimpleTextInfoDialog(TInt aTextIdOne, TInt aRichTextCtlIdOne, TInt aTextIdTwo, TInt aRichTextCtlIdTwo, TBool aSimpleTextResIdOneIsArray, TBool aSimpleTextResIdTwoIsArray)
	: iRichTextCtlIdOne(aRichTextCtlIdOne), iSimpleTextResIdOne(aTextIdOne),
	  iRichTextCtlIdTwo(aRichTextCtlIdTwo), iSimpleTextResIdTwo(aTextIdTwo),
	  iSimpleTextResIdOneIsArray(aSimpleTextResIdOneIsArray), iSimpleTextResIdTwoIsArray(aSimpleTextResIdTwoIsArray),
	  iSetDialogBackground(ETrue)
{
}

void CSimpleTextInfoDialog::PreLayoutDynInitL()
{
	CEikRichTextEditor*		richTextEditor;
	
	if (iRichTextCtlIdOne != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdOne));
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdOne, iSimpleTextResIdOne);
	}

	if (iRichTextCtlIdTwo != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdTwo));
		richTextEditor->Border().SetType(ENone);
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdTwo, iSimpleTextResIdTwo);
	}
}


void CSimpleTextInfoDialog::PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aSimpleTextResId)
{
	iEikonEnv->BusyMsgL(_L("Opening"));
	aRichTextEditor.SetEdwinSizeObserver(this);
	if (iSetDialogBackground)
		aRichTextEditor.SetBackgroundColorL(iEikonEnv->Color(EColorDialogBackground));
	aRichTextEditor.SetSize(aRichTextEditor.Size()); // Set the size of the edwin

		// no scrollbars for us
	aRichTextEditor.CreateScrollBarFrameL(); // Create the scrollbars
	aRichTextEditor.ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, iWantVertScrollbar ? CEikScrollBarFrame::EAuto: CEikScrollBarFrame::EOff);

	ShowTextL(*aRichTextEditor.RichText(), aRichTextCtlId, aSimpleTextResId);

	aRichTextEditor.UpdateAllFieldsL(); // Updates all the fields in the document

	aRichTextEditor.UpdateScrollBarsL();
}


void CSimpleTextInfoDialog::PostLayoutDynInitL()
{
	Layout();
	iEikonEnv->BusyMsgCancel();
}

TBool CSimpleTextInfoDialog::HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize)
{
	if ((aEventType == EEventSizeChanging))
		aEdwin->SetSize(aDesirableEdwinSize);
	return EFalse;
}

void CSimpleTextInfoDialog::ShowTextL(CRichText& aRichText, TInt /*aRichTextCtlId*/, TInt aResId)
{
	if (aResId != -1)
	{
		if ( ((aResId == iSimpleTextResIdOne) && (iSimpleTextResIdOneIsArray)) ||
		     ((aResId == iSimpleTextResIdTwo) && (iSimpleTextResIdTwoIsArray))
		   )
		{
			CDesCArrayFlat* desArray = CEikonEnv::Static()->ReadDesCArrayResourceL(aResId);
			CleanupStack::PushL(desArray);

			CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();

			TInt	count = desArray->Count();
			for (TInt i=0 ;i<count ; i++)
				parser->ParseL(desArray->operator[](i), aRichText);

			CleanupStack::PopAndDestroy(parser);
			CleanupStack::PopAndDestroy(desArray);
		}
		else
		{
			HBufC*		text =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
			ShowSimpleTextL(*text, aRichText);
			CleanupStack::PopAndDestroy(text);
		}
	}
}

void CSimpleTextInfoDialog::ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText)
{
	CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();
	parser->ParseL(aSimpleText, aRichText);

	CleanupStack::PopAndDestroy(parser);
}
