/*******************************************************************
 *
 *	File:		EMameEngine.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifdef __WINS__
#include "EikEnv.h"
#endif

#include "EMameEngine.h"
#include <badesca.h>

const TInt KGameStackSize	= 0x10000;			// 64KB
const TInt KGameHeapSizeMin	= 0x10000;			// 64KB
const TInt KGameHeapSizeMax	= 0x2000000;		// 32 MB

_LIT(KEmameOnC, "c:\\system\\apps\\emame\\");
_LIT(KOptionsFile, "c:\\system\\apps\\emame\\options.ini");
_LIT(KGameRomsFile, "c:\\system\\apps\\emame\\gameroms.ini");
_LIT(KRomDirOnC, "C:\\Documents\\Media Files\\Document\\ROMS\\");
_LIT(KRomDirOnD, "D:\\Media Files\\Document\\ROMS\\");
_LIT(KZipsOnC, "C:\\Documents\\Media Files\\Document\\ROMS\\*.zip");
_LIT(KZipsOnD, "D:\\Media Files\\Document\\ROMS\\*.zip");

extern "C" {
extern const struct GameDriver* drivers[];
}



CInstalledGameList* CInstalledGameList::NewL(TInt aMaxGames)
{
	CInstalledGameList*	self = new(ELeave)CInstalledGameList(aMaxGames);
	CleanupStack::PushL(self);
	self->ConstuctL();
	CleanupStack::Pop(self);
	return self;
}

CInstalledGameList::CInstalledGameList(const CInstalledGameList& aOther)
{
	this->operator =(aOther);
}

CInstalledGameList& CInstalledGameList::operator=(const CInstalledGameList& aOther)
{
	iGames = aOther.iGames;
	Mem::Copy(this->iGameIdxs, aOther.iGameIdxs, sizeof(TInt) * iGames);
	return *this;
}

CInstalledGameList::~CInstalledGameList()
{
	delete[] iGameIdxs;
	iGameIdxs = NULL;
}

TInt CInstalledGameList::GameNo(TInt aGameIndex) const
{
	return (aGameIndex>=0 && aGameIndex<iGames) ?
		iGameIdxs[aGameIndex] : 0;
}

void CInstalledGameList::AddGame(TInt aGameNo)
{
	if (iGames<iMaxGames)
		iGameIdxs[iGames++] = aGameNo;
}

TInt CInstalledGameList::Externalise(RFile& aFile)
{
	TPtrC8	ptr((TUint8*)&iGames, sizeof(TInt));
	TPtrC8	ptr1((TUint8*)iGameIdxs, sizeof(TInt) * iGames);
	TInt err = aFile.Write(ptr);
	if (err == KErrNone)
		err = aFile.Write(ptr1);
	return err;
}

TInt CInstalledGameList::Internalise(RFile& aFile)
{
	iGames = 0;
	TPtr8	ptr((TUint8*)&iGames, sizeof(TInt));
	TInt err = aFile.Read(ptr);
	if (err == KErrNone)
	{
		if (iGames > iMaxGames)
			iGames = iMaxGames;
		TPtr8	ptr1((TUint8*)iGameIdxs, sizeof(TInt) * iGames);
		err = aFile.Read(ptr1);
	}
	return err;
}

CInstalledGameList::CInstalledGameList(TInt aMaxGames) : iMaxGames(aMaxGames)
{
}

void CInstalledGameList::ConstuctL()
{
	iGameIdxs = new(ELeave)TInt[iMaxGames];
}



TInt CGameRunner::ThreadMameGame(TAny* aData)
{  
	CTrapCleanup* cleanup=CTrapCleanup::New();

	CGameRunner*	self = (CGameRunner*) aData;
	TRAPD(err, self->DoRunGameL());

	delete cleanup;

	return(err);
}

CGameRunner::~CGameRunner()
{
}

CGameRunner::CGameRunner(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine) : CActive(CActive::EPriorityStandard), iGameWatcher(aGameWatcher), iMameEngine(aMameEngine)
{
}

CGameRunner* CGameRunner::NewL(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine)
{
	CGameRunner*	self = new(ELeave) CGameRunner(aGameWatcher, aMameEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();			// self
	return self;
}

void CGameRunner::ConstructL()
{
	CActiveScheduler::Add(this);
}

void CGameRunner::RunL()
{
	iGameWatcher.NotifyGameFinished();
}

void CGameRunner::DoCancel()
{
	iThread.LogonCancel(iStatus);
}

void CGameRunner::RunGameL(TInt aGame)
{
	TBuf<32>	threadName;

		/*
		 * Hmm, there is something funny about the game thread on target. Eventhough the
		 * previous game thread has finished (we even called Terminate on it) we cannot
		 * create a new instance of it with the same name (returns with KErrAlreadyExists).
		 * Using unique names gets round that.
		 */
	iGame = aGame;
	threadName.Format(_L("Game%d"), iGameCounter++);
	User::LeaveIfError(iThread.Create(threadName, CGameRunner::ThreadMameGame,  KGameStackSize, KGameHeapSizeMin, KGameHeapSizeMax, this, EOwnerThread));

	iThread.Logon(iStatus);
	SetActive();

	iThread.Resume();

}


void CGameRunner::DoRunGameL()
{
	iMameEngine.RunGameL(iGame);
}


void CGameRunner::TerminateGame()
{
	Cancel();
	ShutdownGame();
}


void CGameRunner::ShutdownGame()
{
	if (iThread.Handle())
	{
#ifdef __WINS__
			/*
			 * Throw in some waits before killing it off. Thread's dying whilst running under the  
			 * may result in a emulator + devstudio freeze.
			 */
		User::After(1000 * 500);
		iMameEngine.KillRunningStdoutConsole();
		User::After(1000 * 500);
#else
		iThread.Terminate(0);
#endif
		iThread.Close();
	}
}


CMameEngine* CMameEngine::NewL(RFs& aFs)
{
	CMameEngine* self = new CMameEngine(aFs);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

CMameEngine::CMameEngine(RFs& aFs) : iFs(aFs)
{
}


CMameEngine::~CMameEngine()
{
}


void CMameEngine::ConstructL()
{
	iFs.MkDirAll(KEmameOnC);
	iFs.MkDirAll(KRomDirOnC);
	iFs.MkDirAll(KRomDirOnD);

	CalcNoOfGames();

	LoadOptions();

	iInstalledGameList = CInstalledGameList::NewL(iNoOfGames);
	LoadInstalledGameList();
}





void CMameEngine::KillRunningStdoutConsole()
{
#ifdef __WINS__
	TInt	id = 0;
	TBool	sentEvent = EFalse;
	RWsSession&		wsSession = CEikonEnv::Static()->WsSession();

		/*
		 * When EPOC's STDLIB shuts down it will put up a "press a key to close console".
		 * In WINS, we'll find the window group belonging to the thread STDOUT and send
		 * it a key event to have it automatically closed.
		 */
	while (id >= 0)
	{
		id = wsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (wsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					TFullName	name;
					name = thread.Name();
					if (name.Compare(_L("STDOUT")) == 0)
					{
						TWsEvent	event;
						TKeyEvent	key;
						
						key.iCode=EKeyEscape;
						key.iScanCode=0;
						key.iModifiers=0;

						event.SetType(EEventKey);
						*event.Key()=key;
						event.SetTimeNow();
						wsSession.SendEventToWindowGroup(id, event);

						sentEvent = ETrue;
					}
					thread.Close();
				}
			}
		}
	}
	if (!sentEvent)
		User::InfoPrint(_L("Please close the task named STDOUT"));
#endif
}



void CMameEngine::RunGameL(TInt aGame)
{
	::RunGame(aGame, iGameOptions, iAppWindowGroupId);
}



void CMameEngine::SetWindowGroupId(TInt aWindowGroupId)
{
	iAppWindowGroupId = aWindowGroupId;
}


TInt CMameEngine::GameNo(const TDesC& aGameName)
{
	TInt	gameNo = 0;
	const struct GameDriver**	game = drivers;
	while(*game)
	{
		TPtrC8	gameName8((TUint8*) (*game)->description);
		TBuf<128>	gameName;
		gameName.Copy(gameName8);
		if (aGameName.Compare(gameName) == 0)
			break;
		game++;
		gameNo++;
	}
	return gameNo;
}

TBool CMameEngine::IsGameCloneOf(const struct GameDriver* aGame)
{
	return (aGame->clone_of && !(aGame->clone_of->flags & NOT_A_DRIVER));
}

const char* CMameEngine::GameName(TInt aGameNo)
{
	const struct GameDriver**	game = drivers;
	return (*(game+aGameNo))->description;
}

const char* CMameEngine::GameShortName(TInt aGameNo)
{
	const struct GameDriver**	game = drivers;
	return (*(game+aGameNo))->name;
}


void CMameEngine::ForceGameExit()
{
	SetForcedExitFromFrontEnd(ETrue);
}


void CMameEngine::ResumeGame()
{
	RSemaphore	semaphore;
	TInt err = semaphore.OpenGlobal(_L("EMAME_SEM"));
	if (!err)
	{
		semaphore.Signal();
		semaphore.Close();
	}
	else
	{
		User::InfoPrint(_L("Could not resume game"));
	}
}

void CMameEngine::DefaultOptions()
{
	iGameOptions.iSoundEnabled = ETrue;
	iGameOptions.iGameSize = TGameOptions::EGameSizeNormal;
}

void CMameEngine::LoadOptions()
{
	TPckg<TGameOptions>	pkg(iGameOptions);

	RFile	file;
	TInt err = file.Open(iFs, KOptionsFile, 0);
	if (!err)
	{
		TInt			version;
		TPckg<TInt>		pkgVersion(version);
		err = file.Read(pkgVersion);
		if (err == KErrNone)
		{
			if (version == KOptionsFileVersion1)
				err = file.Read(pkg);
		}
		file.Close();
	}
	if (err)
	{
		DefaultOptions();
	}
}

TInt CMameEngine::SaveOptions()
{
	TPckgC<TGameOptions>	pkg(iGameOptions);
	RFile	file;
	TInt err = file.Open(iFs, KOptionsFile, EFileWrite);
	if (err)
		err = file.Create(iFs, KOptionsFile, EFileWrite);
	if (!err)
	{
		TPckgC<TInt>	pkgVersion(KOptionsFileVersion1);
		err = file.Write(pkgVersion);
		if (err == KErrNone)
			err = file.Write(pkg);
		file.Close();
	}
	return err;
}


void CMameEngine::SetGameOptions(const TGameOptions& aGameOptions)
{
	iGameOptions = aGameOptions;
	if (SaveOptions() == KErrNone)
		User::InfoPrint(_L("Options saved"));
}

const struct GameDriver** CMameEngine::GameDrivers()
{
	return drivers;
}

const char* CMameEngine::CpuName(TInt aCpuType)
{
	return cputype_name(aCpuType);
}

const CInstalledGameList& CMameEngine::InstalledGameList()
{
	return *iInstalledGameList;
}

TInt CMameEngine::SaveInstalledGameList()
{
	RFile	file;
	TInt err = file.Open(iFs, KGameRomsFile, EFileWrite);
	if (err)
		err = file.Create(iFs, KGameRomsFile, EFileWrite);
	if (!err)
	{
		err = iInstalledGameList->Externalise(file);
		file.Close();
	}
	return err;
}

void CMameEngine::LoadInstalledGameList()
{
	RFile	file;
	TInt err = file.Open(iFs, KGameRomsFile, 0);
	if (!err)
	{
		err = iInstalledGameList->Internalise(file);
		file.Close();
	}
}

void CMameEngine::SetInstalledGameList(const CInstalledGameList& aInstalledGameList)
{
	*iInstalledGameList = aInstalledGameList;
	if (SaveInstalledGameList() == KErrNone)
	{
		if (iInstalledGameList->NoOfGames() > 0)
			User::InfoPrint(_L("Game list saved"));
	}
}


void CMameEngine::CalcNoOfGames()
{
	iNoOfGames = 0;
	const struct GameDriver**	game = GameDrivers();
	while(*game++)
		iNoOfGames++;
}

void DestroyCachedZipList();
void SetCachedZipList(CDesCArrayFlat* aCachedZipList);

void CMameEngine::DestroyZipList()
{
	DestroyCachedZipList();
}

void CMameEngine::CreateZipList()
{
	CDesCArrayFlat* desArray = new(ELeave)CDesCArrayFlat(10);
	CleanupStack::PushL(desArray);
	CDir*	entryList = NULL;
	if (iFs.GetDir(KZipsOnC, 0, 0, entryList) == KErrNone)
	{
		CleanupStack::PushL(entryList);
		for (TInt i=0 ; i<entryList->Count() ; i++)
		{
			const TEntry& entry = (*entryList)[i];
			TParsePtrC	parse(entry.iName);
			TPtrC	name(parse.Name());
			desArray->AppendL(name);
		}
		CleanupStack::PopAndDestroy(entryList);
	}
	entryList = NULL;
	if (iFs.GetDir(KZipsOnD, 0, 0, entryList) == KErrNone)
	{
		CleanupStack::PushL(entryList);
		for (TInt i=0 ; i<entryList->Count() ; i++)
		{
			const TEntry& entry = (*entryList)[i];
			TParsePtrC	parse(entry.iName);
			TPtrC	name(parse.Name());
			desArray->AppendL(name);
		}
		CleanupStack::PopAndDestroy(entryList);
	}
	CleanupStack::Pop();

	SetCachedZipList(desArray);
}

