/*******************************************************************
 *
 *	File:		Controls.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 200s, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#define IgnoreEventsUntilNextPointerUp() IgnoreEventsUntilNextPointerUp();friend class CCoeControlContainer;
#include <coecntrl.h>

#include "Controls.h"

#include <eikenv.h>
#include <eiklabel.h>


/***********************************************************
 *
 * CBitmappedCoeControl
 *
 ***********************************************************/

void CBitmappedCoeControl::DrawControl()
{
	if (iBitmap)
	{
		iBitmapContext->Clear();

		// DrawControl will draw relative to its Position().
		// when drawing to the bitmap gc Position() should be (0,0)
		TPoint	position = iPosition;
		iPosition = TPoint(0,0);
		DrawControl(*iBitmapContext, *iBitmapDevice);
		iPosition = position;
	}
}

void CBitmappedCoeControl::SizeChanged()
{
		/*
		 * Destroy old bitmap and try to create a new one.
		 * If the new one is created, call DrawControl which will
		 * draw it that bitmap (we'll wait until we get a redraw
		 * to blit it to screen); if it can't be created we do nothing
		 * (from the redraw we call DrawControl with the window's gc)
		 */
	DestroyBitmap();
	TRAPD(err, CreateBitmapL(Size()));
	if (err)
		DestroyBitmap();
	else
		DrawControl();
}

void CBitmappedCoeControl::DestroyBitmap()
{
	delete iBitmapContext;
	iBitmapContext = NULL;
	delete iBitmapDevice;
	iBitmapDevice = NULL;
	delete iBitmap;
	iBitmap = NULL;
}

void CBitmappedCoeControl::CreateBitmapL(const TSize& aSize)
{
	iBitmap = new (ELeave) CFbsBitmap;
	iBitmap->Create(aSize, iEikonEnv->ScreenDevice()->DisplayMode());

	iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap);
	iBitmapDevice->CreateContext(iBitmapContext);
}

void CBitmappedCoeControl::Draw(const TRect& aRect) const
{
    CWindowGc& gc=SystemGc();

	if (iBitmap)
		gc.BitBlt(aRect.iTl, iBitmap, TRect(aRect.iTl - Position(), aRect.Size()));
	else
	{
		CWsScreenDevice* screenDevice = iCoeEnv->ScreenDevice();
		DrawControl(gc, *screenDevice);
	}
}

/***********************************************************
 *
 * ProgressControl
 *
 ***********************************************************/


CProgressControl::CProgressControl() : iProgressColor(KRgbBlue)
{
}

void CProgressControl::DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& /*aBitmapDevice*/) const
{
	TRect	rect = Rect();

		/*
		 * Draw black outer-rect
		 */
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	aBitmapContext.DrawRect(rect);

		/*
		 * Draw progress bar
		 */
	aBitmapContext.SetPenColor(iProgressColor);
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetBrushColor(iProgressColor);
	rect.Shrink(1, 1);
	TInt fullWidth = rect.Size().iWidth;
	TInt width = (fullWidth * iProgressPercent) / 100;
	rect.iBr.iX -= (fullWidth - width);
	aBitmapContext.DrawRect(rect);

		/*
		 * Clear the remainder of the bar
		 */
	aBitmapContext.SetPenColor(KRgbWhite);
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetBrushColor(KRgbWhite);
	rect.iTl.iX = rect.iBr.iX+1;
	rect.iBr.iX = (fullWidth - width);
	aBitmapContext.DrawRect(rect);

	TBuf<32>	text;
	text.Format(_L("%d%%"), iProgressPercent);
	
	const CFont*	font = iCoeEnv->NormalFont();
	TInt	ascent = font->AscentInPixels();
	TRect	textRect = Rect();
	textRect.Shrink(1, 1);
	textRect.Shrink(0, (textRect.Size().iHeight - ascent)/2);

	aBitmapContext.UseFont(font);
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.SetPenSize(TSize(1,1));
	aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	aBitmapContext.DrawText(text, textRect, ascent, CGraphicsContext::ECenter);
#ifdef __WINS__
	RDebug::Print(_L("Per=%d, width=%d"), iProgressPercent, width);
#endif
}

void CProgressControl::SetProgress(TInt aPercent, TBool aDrawNow)
{
	if (aPercent != iProgressPercent)
	{
		iProgressPercent = aPercent;
		CBitmappedCoeControl::DrawControl();
		if (aDrawNow)
			DrawNow();
		else
			DrawDeferred();
	}
}
